

#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#endif

#include "global.h"

#include "vgev_draw.h"


/* ****************************************************************
**************************************************************** */
WVSmap::WVSmap() {

   map_avail = 0;
   borders = landmass = NULL;
   bordercoord = landcoord = NULL;
   landedge = NULL;
   landcount = NULL;
   landclip = NULL;
   landpcount = 0;
}


/* ****************************************************************
**************************************************************** */
void WVSmap::dest() {

   if (landcoord) {
      delete [] landcoord;
      landcoord = NULL;
   }

   if (landedge) {
      delete [] landedge;
      landedge = NULL;
   }

   if (landcount) {
      delete [] landcount;
      landcount = NULL;
   }

   if (landclip) {
      delete [] landclip;
      landclip = NULL;
   }

   if (bordercoord) {
      delete [] bordercoord;
      bordercoord = NULL;
   }

}


/* ****************************************************************
**************************************************************** */
WVSmap::~WVSmap() {

   dest();
}


/* ****************************************************************
**************************************************************** */
void WVSmap::read_map(char *landname, char *bordername) {

   file_loader *loader;

   dest();
   
   loader = (file_loader *)global_resource_manager->find_resource(RESOURCE_POLYGON_LOADER, landname, NULL);

   if (loader) {
      if (loader->read_data(landname)) {
         landmass = (polytype *)loader->extract(FILETYPE_SPG, landname);
         landcoord = new vector2i[landmass->countvertex];
         landedge = new int[landmass->countedge];
         landcount = new int[landmass->countedge];
         landclip = new int[landmass->countvertex];
      }

      loader->dest(DYNAMIC_LOADER ? 0x01 : 0x00);
   }

   loader = (file_loader *)global_resource_manager->find_resource(RESOURCE_POLYGON_LOADER, bordername, NULL);

   if (loader) {
      if (loader->read_data(bordername)) {
         borders = (polytype *)loader->extract(FILETYPE_SPG, bordername);
         bordercoord = new vector2i[borders->countvertex];
      }

      loader->dest(DYNAMIC_LOADER ? 0x01 : 0x00);
   }

   map_avail = landmass || borders;
}


/* ****************************************************************************
**************************************************************************** */
void WVSmap::render(canvas *tgev) {

   polygon_object_type *pot;
   int i, j;
   int ilat,ilon;

   if (map_avail) {
      glEnableClientState(GL_VERTEX_ARRAY);

      glClearColor(0.0980f,0.0980f,0.4392f,1.0f);

      glClear(GL_COLOR_BUFFER_BIT);

      if (landmass) {
         tgev->tpColor3ub(139, 71, 38, 0);
       
         pot = (polygon_object_type *)landmass->query_data();

         glVertexPointer(2, GL_INT, 8, landcoord);

         for (i=j=0; i<landpcount; j += landcount[i], i++)
            glDrawElements(GL_POLYGON, landcount[i], GL_UNSIGNED_INT, &landedge[j]);
      }

      if (borders) {
         tgev->tpColor3ub(139, 0, 0, 1);
       
         pot = (polygon_object_type *)borders->query_data();

         glVertexPointer(2, GL_INT, 8, bordercoord);

         for (i=0; i<borders->countobject; i++)
            glDrawElements(GL_LINE_STRIP, pot->flist[i].polynum, GL_UNSIGNED_INT, pot->flist[i].edgeptr);
      }

      glDisableClientState(GL_VERTEX_ARRAY);
   }

   tgev->tpColor3ub(179, 179, 179, 1);

   // draw lat_lon grid
   for (i=0; i < 703; i+=37) {

      glBegin(GL_LINE_STRIP);

         for (j=0; j<37; j++)
            glVertex2iv(llcoord[i+j]);

      glEnd();

   }

   for (j=0; j<37; j++) {

      glBegin(GL_LINE_STRIP);

         for (i=0; i < 703; i+=37)
            glVertex2iv(llcoord[i+j]);

      glEnd();

   }

   // tick every degree
   if (tgev->get_scale() >= 8) {
      glBegin(GL_LINES);
         i = 0;
         for (ilat = -90; ilat < 90; ilat += 10) {
            for (ilon=-180; ilon <180; ilon += 10) {

               for (j=1; j<10; j++, i++) {
                  glVertex2iv(tickcoord[i]);
                  glVertex2i(tickcoord[i][0]+5,tickcoord[i][1]);
               }

               for (j=1; j<10; j++, i++) {
                  glVertex2iv(tickcoord[i]);
                  glVertex2i(tickcoord[i][0],tickcoord[i][1]+5);
               }

            }

         }

      glEnd();
   }

   glEnd();
}


/* ****************************************************************************
**************************************************************************** */
void WVSmap::preprocess_map(canvas *tgev) {

   int i, j, k, l;
   int ilat, ilon;
   polygon_object_type *pot;

   if (landmass) {
      pot = (polygon_object_type *)landmass->query_data();

      for (i=0;i<landmass->countvertex;i++)
         landclip[i] = tgev->clip(pot->vlist[i][0], pot->vlist[i][1],&landcoord[i][0], &landcoord[i][1]);

      landpcount = l = 0;
      for (i=0; i<landmass->countobject; i++) {
         k = landclip[pot->flist[i].edgeptr[0]];
         for (j=1; j<pot->flist[i].polynum; j++)
            k &= landclip[pot->flist[i].edgeptr[j]];

         if (k)
            continue;
 
         landcount[landpcount] = pot->flist[i].polynum;
         landpcount++;
         for (j=0; j<pot->flist[i].polynum; j++, l++)
            landedge[l] = pot->flist[i].edgeptr[j];
      }

   }

   if (borders) {
      pot = (polygon_object_type *)borders->query_data();

      for (i=0;i<borders->countvertex;i++)
         tgev->clip(pot->vlist[i][0], pot->vlist[i][1],&bordercoord[i][0], &bordercoord[i][1]);
   }

   i = 0;
   // ticks every 10 degrees
   for (ilat = -90; ilat <= 90; ilat += 10)
      for (ilon=-180; ilon <=180; ilon += 10, i++)
         tgev->clip((double)ilon, (double)ilat,&llcoord[i][0],&llcoord[i][1]);

   // tick every degree
   if (tgev->get_scale() >= 8) {
      i = 0;
      for (ilat = -90; ilat < 90; ilat += 10)
         for (ilon=-180; ilon <180; ilon += 10) {
            for (j=1; j<10; j++, i++)
               tgev->clip((double)ilon,(double)(ilat+j),&tickcoord[i][0],&tickcoord[i][1]);
            for (j=1; j<10; j++, i++)
               tgev->clip((double)(ilon+j),(double)ilat,&tickcoord[i][0],&tickcoord[i][1]);
         }

   }


}


/* ****************************************************************************
**************************************************************************** */
void GreyWVSmap::render(canvas *tgev) {

   polygon_object_type *pot;
   int i, j;
   int ilat,ilon;

   if (map_avail) {
      glEnableClientState(GL_VERTEX_ARRAY);

      glClearColor(0.4392f,0.4392f,0.4392f,1.0f);

      glClear(GL_COLOR_BUFFER_BIT);

      if (landmass) {
         tgev->tpColor3ub(139, 71, 38, 0);
       
         pot = (polygon_object_type *)landmass->query_data();

         glVertexPointer(2, GL_INT, 8, landcoord);

         for (i=j=0; i<landpcount; j += landcount[i], i++)
            glDrawElements(GL_POLYGON, landcount[i], GL_UNSIGNED_INT, &landedge[j]);
      }

      if (borders) {
         tgev->tpColor3ub(139, 0, 0, 1);
       
         pot = (polygon_object_type *)borders->query_data();

         glVertexPointer(2, GL_INT, 8, bordercoord);

         for (i=0; i<borders->countobject; i++)
            glDrawElements(GL_LINE_STRIP, pot->flist[i].polynum, GL_UNSIGNED_INT, pot->flist[i].edgeptr);
      }

      glDisableClientState(GL_VERTEX_ARRAY);
   }

   tgev->tpColor3ub(179, 179, 179, 1);

   // draw lat_lon grid
   for (i=0; i < 703; i+=37) {

      glBegin(GL_LINE_STRIP);

         for (j=0; j<37; j++)
            glVertex2iv(llcoord[i+j]);

      glEnd();

   }

   for (j=0; j<37; j++) {

      glBegin(GL_LINE_STRIP);

         for (i=0; i < 703; i+=37)
            glVertex2iv(llcoord[i+j]);

      glEnd();

   }

   // tick every degree
   if (tgev->get_scale() >= 8) {
      glBegin(GL_LINES);
         i = 0;
         for (ilat = -90; ilat < 90; ilat += 10) {
            for (ilon=-180; ilon <180; ilon += 10) {

               for (j=1; j<10; j++, i++) {
                  glVertex2iv(tickcoord[i]);
                  glVertex2i(tickcoord[i][0]+5,tickcoord[i][1]);
               }

               for (j=1; j<10; j++, i++) {
                  glVertex2iv(tickcoord[i]);
                  glVertex2i(tickcoord[i][0],tickcoord[i][1]+5);
               }

            }

         }

      glEnd();
   }

   glEnd();
}


