

#include <string.h>

#include "vpixcon.h"


/* **************************************************************
************************************************************** */
pix_data::pix_data(basic_platform *x) : platform_data(x) {

// set in basic_platform
//   x->platform_flag |= PLATFORM_FLAG_DEAD_RECOG;
   player = NULL;
}


/* **************************************************************
************************************************************** */
pix_data::~pix_data() {

   quark **btr;
  
   if (!player)
      return;
      
   for (btr = (quark **)&pixcon_object_list; *btr && *btr != player; btr = (quark **)&(*btr)->next);
      
   if (*btr)
      *btr = (quark *)player->next;

   player->next = NULL;
   delete player;
}


/* **************************************************************
************************************************************** */
void pix_data::label(dbl_llist_manager *label_manager) {

   vector4f s;
   labeltype *ptr;

   if (!player || !(player->flags & QUARK_FLAG_VISIBLE) || !(player->flags & QUARK_FLAG_ACTIVE))
      return;

   copyarray3(s, player->state.center);
   s[3] = 1.0;
   
   matvecmulto(current_camera->transform, s);
   current_camera->map2screen(s);

   label_manager->insert(ptr = (labeltype *)vmemman.pop(MM_LABEL), NULL);
   ptr->set_label(player->name.string);
   copyarray2(ptr->pos, s);
}


/* **************************************************************
************************************************************** */
void pix_data::update(double *machine_time) {

   vector4f mx[4];
   vector4d start, end, midpt;
   vector3d ray;
   vector4d p, normal;
   double z = parent->plat_clone.pos[2];
   double limit = 1000000.0;
   
   if (!player || !(parent->platform_flag & PLATFORM_FLAG_MOVEMENT))
      return;

   init_mx(mx);

   rotate_mx_y(mx, (float)parent->plat_clone.rpy[0]);
   rotate_mx_x(mx, (float)parent->plat_clone.rpy[1]);
   rotate_mx_z(mx, (float)parent->plat_clone.rpy[2]);

   // smack land base vehicles to the terrain
   if (!(parent->platform_flag & PLATFORM_FLAG_FLY) && terrain_tree) {
      copyarray2(start, parent->plat_clone.pos);
      start[2] = 1000000.0;
      start[3] = 1.0;
      
      copyarray2(end, parent->plat_clone.pos);
      end[2] = -1000000.0;
      end[3] = 1.0;

      copyarray2(midpt, parent->plat_clone.pos);
      midpt[2] = 0;
      midpt[3] = 1.0;
            
      ray[0] = ray[1] = 0; ray[2] = -1.0;
      
      if (point_terrain_collision(terrain_tree, NULL, start, end, midpt, &limit, ray, p, normal))
         z = p[2] + CORRECT;
   }
   
   mx[0][3] = (float)parent->plat_clone.pos[0];   
   mx[1][3] = (float)parent->plat_clone.pos[1];   
   mx[2][3] = (float)z;   

   player->whereami(pixcon_object_list, NULL, mx);
   parent->platform_flag &= ~PLATFORM_FLAG_MOVEMENT;
}


/* **************************************************************
************************************************************** */
void pix_data::render(iface *isim) {

   if (player) {
      player->flags |= QUARK_FLAG_VISIBLE;
      player->update(NULL);
   }

}


/* **************************************************************
************************************************************** */
int pix_data::init(iface *isim, correlate *basis, symbol_correlation_type *str) {

   string_type token;
   vector4f mx[4];
   int i;
   geneological_type ani_param;
   
   token.stringcpy(&basis->objname);
   
   for (i=token.stringlen()-1; i>-1; i--)
      if (token.string[i] == '.') {
         token.string[i] = 0;
	 token.recalc();
	 break;
      }
      
   token.stringcat(".atm");

   player = load_atom(token.string);

   if (!player) {
      printf("Failed to load player %d %s\n", parent->id, token.string);
      return 0;
   }
   
   player->name.stringcpy(&str->sym_label);

   ani_param.tree = player;
   ani_param.parent = NULL;
   player->preprocess(&ani_param);
   
   init_mx(mx);
   player->begin(pixcon_object_list, NULL, mx);
   player->update(NULL);
   
   return 1;
}


/* **************************************************************
************************************************************** */
int pix_data::perform(iface *isim, int task, void *data) {

//   perform_bounding_sphere_type *pbst;
   
   switch (task) {
      case PERFORM_PLATFORM_DEACTIVATE:
         if (player)
	    player->flags &= ~QUARK_FLAG_ACTIVE;

         return 1;

/*
      case PERFORM_QUERY_BOUNDING_SPHERE:
         pbst = (perform_bounding_sphere_type *)data;
	 
         if (!player || (player->old_state.state_flags & STATE_MASK_TREE) != STATE_FLAG_TSPHERE) {
            pbst->pos[0] = pbst->pos[1] = pbst->pos[2] = 0;
            pbst->radius = 0;
            return 1;
         }

         copyarray3(pbst->pos, player->old_state.tree_center);
         pbst->radius = player->old_state.tree_radius;
         return 1;
*/
	 
      default:
         break;
   }
   
   return platform_data::perform(isim, task, data);
}

