

#include "camera_ai.h"


/* ============================================================================
============================================================================ */
void camera_ai::init() {

   mode = PS_CAMERA_MODE_FOLLOW;
   rpy[0] = rpy[1] = rpy[2] = xyz[0] = xyz[1] = xyz[2] = 0;
   orbit_rad = 0;
   speed = 0;
   attach_object = NULL;
   init_mx(xmx);
}
      
      
/* ============================================================================
============================================================================ */
void camera_ai::set_mode(unsigned int m) {

   mode = m;
}


/* ============================================================================
   grab the camera's xyz, pitch and yaw, and store it for future use
============================================================================ */
void camera_ai::set_stat() {

   float r, p, y;
   
   vector4f mx[4];

   copymx4x4(mx, xmx);

   xyz[0] = mx[0][3];
   xyz[1] = mx[1][3];
   xyz[2] = mx[2][3];

   rotate_mx_x(mx, -HALFPI);
   decompose_yyxpzr(mx, &r, &p, &y);
   rpy[0] = 0;
   rpy[1] = p;
   rpy[2] = y;
}


/* ============================================================================
============================================================================ */
void camera_ai::face_target() {

   vector4f mx[4];
   double t;
   vector3f y;
   double xd, yd, zd;

   init_mx(mx);

   rotate_mx_y(mx, (float)attach_object->plat_clone.rpy[0]);
   rotate_mx_x(mx, (float)attach_object->plat_clone.rpy[1]);
   rotate_mx_z(mx, (float)attach_object->plat_clone.rpy[2]);

   // up vector
   y[0] = mx[0][2];
   y[1] = mx[1][2];
   y[2] = mx[2][2];

   // calc front face vector
   copyarray3(mx[1], (float)attach_object->plat_clone.pos);

   xd = mx[1][0];
   yd = mx[1][1];
   zd = mx[1][2];
   nix_tunneling(&xd, &yd, &zd);
   mx[1][2] = zd;

   mx[1][0] -= xmx[0][3];
   mx[1][1] -= xmx[1][3];
   mx[1][2] -= xmx[2][3];

   normalize3(mx[1]);
  
   // calc right vector 
   t = dotproduct3(mx[1], y);

   if (fabs(t) > ICORRECT) {
      y[0] = xmx[0][2];
      y[1] = xmx[1][2];
      y[2] = xmx[2][2];
   }

   xproduct(mx[0], y, mx[1]);
   normalize3(mx[0]);

   xproduct(mx[2], mx[1], mx[0]);

   xmx[0][0] = mx[0][0];
   xmx[0][1] = mx[1][0];
   xmx[0][2] = mx[2][0];

   xmx[1][0] = mx[0][1];
   xmx[1][1] = mx[1][1];
   xmx[1][2] = mx[2][1];

   xmx[2][0] = mx[0][2];
   xmx[2][1] = mx[1][2];
   xmx[2][2] = mx[2][2];
}


/* ============================================================================
 ============================================================================ */
void camera_ai::freefly(iface *iman, user_io *userinput, timer_type *timer) {

   double scale = 0;
   vector3d dir;
   double d;
   plat_stat_type pc;
   TIME_STRUCT t;
   
   process_user_io(userinput, timer);

   scale = speed * timer->speedscale;

   if (rpy[1] > HALFPI)
      rpy[1] = HALFPI-CORRECT;
   else if (rpy[1] < -HALFPI+CORRECT)
      rpy[1] = -HALFPI+CORRECT;

   init_mx(xmx);
   rotate_mx_y(xmx, (float)rpy[0]);
   rotate_mx_x(xmx, (float)rpy[1]);
   rotate_mx_z(xmx, (float)rpy[2]);

   dir[0] = xmx[0][1];
   dir[1] = xmx[1][1];
   dir[2] = xmx[2][1];
   normalize3d(dir);
   
   xyz[0] += scale * dir[0];
   xyz[1] += scale * dir[1];
   xyz[2] += scale * dir[2];

   d = xyz[2];
   nix_tunneling(&xyz[0], &xyz[1], &d);
   xyz[2] = d;
   
   xmx[0][3] = (float)xyz[0];
   xmx[1][3] = (float)xyz[1];
   xmx[2][3] = (float)xyz[2];

   if (iman && (iman->interface_flag & INTERFACE_FLAG_MINICREW)) {
      copyarray3(pc.pos, xyz);
      copyarray3(pc.rpy, rpy);   
      GETTIME(&t);
      pc.machine_time = TIME2DOUBLE(t);
      iman->mcp(&pc);
   }

}   


/* ============================================================================
 ============================================================================ */
void camera_ai::followfly(timer_type *timer) {

   vector3d pos, old;

   old[0] = xmx[0][3];
   old[1] = xmx[1][3];
   old[2] = xmx[2][3];

   init_mx(xmx);

   rotate_mx_z(xmx, camera_azimuth);

   rotate_mx_y(xmx, (float)attach_object->plat_clone.rpy[0]);
   rotate_mx_x(xmx, (float)attach_object->plat_clone.rpy[1]);
   rotate_mx_z(xmx, (float)attach_object->plat_clone.rpy[2]);

   copyarray3(pos, attach_object->plat_clone.pos);
   if (!(attach_object->platform_flag & PLATFORM_FLAG_FLY))
      clamp_tunneling(&pos[0], &pos[1], &pos[2]);

   xmx[0][3] = (float)(pos[0] - camera_dist*xmx[0][1] + camera_height*xmx[0][2]);
   xmx[1][3] = (float)(pos[1] - camera_dist*xmx[1][1] + camera_height*xmx[1][2]);
   xmx[2][3] = (float)(pos[2] - camera_dist*xmx[2][1] + camera_height*xmx[2][2]);

   face_target();

   old[0] -= xmx[0][3];
   old[1] -= xmx[1][3];
   old[2] -= xmx[2][3];

   speed = dotproduct3(old, old);
   speed = speed < CORRECT ? 0 : sqrt(speed)/timer->speedscale;
}


/* ============================================================================
 ============================================================================ */
void camera_ai::orbitfly(timer_type *timer) {

   init_mx(xmx);

   orbit_rad += timer->speedscale * camera_angular_velocity;

   rotate_mx_y(xmx, orbit_rad);
   rotate_mx_x(xmx, -HALFPI);
   rotate_mx_y(xmx, PI);

//asdf negated yaw - havent fixed this code
   rotate_mx_y(xmx, (float)attach_object->plat_clone.rpy[0]);
   rotate_mx_x(xmx, (float)attach_object->plat_clone.rpy[1]);
   rotate_mx_z(xmx, (float)attach_object->plat_clone.rpy[2]);

   xmx[0][3] = (float)(attach_object->plat_clone.pos[0] - camera_dist*xmx[0][2] + camera_height*xmx[0][1]);
   xmx[1][3] = (float)(attach_object->plat_clone.pos[1] - camera_dist*xmx[1][2] + camera_height*xmx[1][1]);
   xmx[2][3] = (float)(attach_object->plat_clone.pos[2] - camera_dist*xmx[2][2] + camera_height*xmx[2][1]);

   face_target();
}   


/* ============================================================================
 ============================================================================ */
void camera_ai::mimicfly(timer_type *timer) {

   double x, y, z;
   
   init_mx(xmx);

   rotate_mx_x(xmx, -HALFPI);
   rotate_mx_y(xmx, PI);

// asdf negated yaw - havent fixed this code
   rotate_mx_y(xmx, (float)attach_object->plat_clone.rpy[0]);
   rotate_mx_x(xmx, (float)attach_object->plat_clone.rpy[1]);
   rotate_mx_z(xmx, (float)attach_object->plat_clone.rpy[2]);

   xmx[0][3] = (float)(attach_object->plat_clone.pos[0] - camera_dist*xmx[0][2]);
   xmx[1][3] = (float)(attach_object->plat_clone.pos[1] - camera_dist*xmx[1][2]);
   xmx[2][3] = (float)(attach_object->plat_clone.pos[2] - camera_dist*xmx[2][2]);

   x = xmx[0][3];
   y = xmx[1][3];
   z = xmx[2][3];
   nix_tunneling(&x, &y, &z);
   xmx[2][3] = z;
}   


/* ============================================================================
 ============================================================================ */
void camera_ai::cockpitfly(timer_type *timer) {

   double x, y, z;
   
   init_mx(xmx);

   rotate_mx_x(xmx, -HALFPI);
   rotate_mx_y(xmx, PI);

//asdf negated yaw - havent fixed this code
   rotate_mx_y(xmx, (float)attach_object->plat_clone.rpy[0]);
   rotate_mx_x(xmx, (float)attach_object->plat_clone.rpy[1]);
   rotate_mx_z(xmx, (float)attach_object->plat_clone.rpy[2]);

   xmx[0][3] = (float)attach_object->plat_clone.pos[0];
   xmx[1][3] = (float)attach_object->plat_clone.pos[1];
   xmx[2][3] = (float)attach_object->plat_clone.pos[2];

   x = xmx[0][3];
   y = xmx[1][3];
   z = xmx[2][3];
   nix_tunneling(&x, &y, &z);
   xmx[2][3] = z;
}   


/* ============================================================================
 ============================================================================ */
void camera_ai::animate_camera(iface *iman, user_io *userinput, timer_type *timer) {

   basic_platform *ptr;

   // make sure object pointer is valid
   if (iman && attach_object) {
      for (ptr=(basic_platform *)iman->head; ptr; ptr=(basic_platform *)ptr->next)
         if (ptr == attach_object)
            break;

      attach_object = ptr;
   }

   if (!attach_object) {
      freefly(iman, userinput, timer);
      return;
   }

   speed = 0;

   switch (mode) {
      case PS_CAMERA_MODE_FOLLOW:
         followfly(timer);
         set_stat();
         return;
	 
      case PS_CAMERA_MODE_ORBIT:
         orbitfly(timer);
         set_stat();
         return;
	 
      case PS_CAMERA_MODE_MIMIC:
         mimicfly(timer);
         set_stat();
         return;
	 
      case PS_CAMERA_MODE_COCKPIT:
         cockpitfly(timer);
         set_stat();
         return;
	 
      case PS_CAMERA_MODE_FREEFLY:
      default:
         freefly(iman, userinput, timer);
         return;
   }
   
}


/* ============================================================================
============================================================================ */
void camera_ai::process_user_io(user_io *userinput, timer_type *timer) {

   double dorient = HALFPI * timer->speedscale;

   // mouse
   if (controller == CONTROLMOUSE || controller == CONTROLKEYBOARD) {
      if (userinput->translate(KEYSTROKE_FORWARD))
         speed += controller_speed[CONTROLMOUSE];
      else if (userinput->translate(KEYSTROKE_BACKWARD))
         speed -= controller_speed[CONTROLMOUSE];
      else if (userinput->translate_stroke(KEYSTROKE_STOP))
         speed = 0;
   }

   // joystick
   else if (userinput->translate(KEYSTROKE_FIRE_PRIMARY))
      speed += controller_speed[CONTROLJOYSTICK];
   else if (userinput->translate(KEYSTROKE_FIRE_SECONDARY))
      speed -= controller_speed[CONTROLJOYSTICK];
   else if (userinput->translate_stroke(KEYSTROKE_STOP))
      speed = 0;

   // bank/roll based on yaw
   rpy[0] = -ASIN(0.5*userinput->control[AXIS_YAW]);
   rpy[1] -= userinput->control[AXIS_PITCH]*dorient;
   rpy[2] += userinput->control[AXIS_YAW]*dorient;
}
